/*
 * Fabric3
 * Copyright (c) 2009 Metaform Systems
 *
 * Fabric3 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version, with the
 * following exception:
 *
 * Linking this software statically or dynamically with other
 * modules is making a combined work based on this software.
 * Thus, the terms and conditions of the GNU General Public
 * License cover the whole combination.
 *
 * As a special exception, the copyright holders of this software
 * give you permission to link this software with independent
 * modules to produce an executable, regardless of the license
 * terms of these independent modules, and to copy and distribute
 * the resulting executable under terms of your choice, provided
 * that you also meet, for each linked independent module, the
 * terms and conditions of the license of that module. An
 * independent module is a module which is not derived from or
 * based on this software. If you modify this software, you may
 * extend this exception to your version of the software, but
 * you are not obligated to do so. If you do not wish to do so,
 * delete this exception statement from your version.
 *
 * Fabric3 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the
 * GNU General Public License along with Fabric3.
 * If not, see <http://www.gnu.org/licenses/>.
*/

package org.fabric3.binding.jms.runtime.host;

import java.net.URI;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import javax.jms.ConnectionFactory;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageListener;
import javax.jms.Session;
import javax.transaction.TransactionManager;

import org.osoa.sca.annotations.Destroy;
import org.osoa.sca.annotations.EagerInit;
import org.osoa.sca.annotations.Init;
import org.osoa.sca.annotations.Reference;
import org.osoa.sca.annotations.Service;

import org.fabric3.api.annotation.Monitor;
import org.fabric3.binding.jms.common.TransactionType;
import org.fabric3.binding.jms.runtime.container.AdaptiveMessageContainer;
import org.fabric3.binding.jms.runtime.container.MessageContainerMonitor;
import org.fabric3.host.work.WorkScheduler;
import org.fabric3.spi.event.EventService;
import org.fabric3.spi.event.Fabric3EventListener;
import org.fabric3.spi.event.RuntimeStart;

/**
 * JmsHost implementation that registers JMS MessageListeners with an AdaptiveMessageContainer to receive messages and dispatch them to a service
 * endpoint.
 *
 * @version $Rev: 7619 $ $Date: 2009-09-12 16:15:15 +0200 (Sat, 12 Sep 2009) $
 */
@EagerInit
@Service(JmsHost.class)
public class JmsHostImpl implements JmsHost, Fabric3EventListener<RuntimeStart> {
    private Map<URI, AdaptiveMessageContainer> containers = new ConcurrentHashMap<URI, AdaptiveMessageContainer>();
    private boolean started;
    private EventService eventService;
    private WorkScheduler scheduler;
    private TransactionManager tm;
    private MessageContainerMonitor containerMonitor;
    private HostMonitor monitor;

    public JmsHostImpl(@Reference EventService eventService,
                       @Reference WorkScheduler scheduler,
                       @Reference TransactionManager tm,
                       @Monitor MessageContainerMonitor containerMonitor,
                       @Monitor HostMonitor monitor) {
        this.eventService = eventService;
        this.scheduler = scheduler;
        this.tm = tm;
        this.containerMonitor = containerMonitor;
        this.monitor = monitor;
    }

    @Init
    public void init() {
        eventService.subscribe(RuntimeStart.class, this);
    }

    @Destroy
    public void destroy() throws JMSException {
        for (AdaptiveMessageContainer container : containers.values()) {
            container.stop();
        }
        for (AdaptiveMessageContainer container : containers.values()) {
            container.shutdown();
        }
        started = false;
    }

    public void onEvent(RuntimeStart event) {
        // start receiving messages after the runtime has started
        for (Map.Entry<URI, AdaptiveMessageContainer> entry : containers.entrySet()) {
            try {
                entry.getValue().initialize();
                monitor.registerListener(entry.getKey());
            } catch (JMSException e) {
                // TODO This should send an asynchronous notify
                monitor.error("Error starting service listener for " + entry.getKey(), e);
            }
        }
        started = true;
    }

    public boolean isRegistered(URI serviceUri) {
        return containers.containsKey(serviceUri);
    }

    public void register(ListenerConfiguration configuration) throws JMSException {
        Destination destination = configuration.getDestination();
        MessageListener listener = configuration.getMessageListener();
        ConnectionFactory factory = configuration.getFactory();
        TransactionType type = configuration.getType();
        URI serviceUri = configuration.getUri();
        AdaptiveMessageContainer container = new AdaptiveMessageContainer(destination, listener, factory, scheduler, tm, containerMonitor);
        if (TransactionType.GLOBAL == type) {
            container.setTransactionType(TransactionType.GLOBAL);
            container.setAcknowledgeMode(Session.AUTO_ACKNOWLEDGE);
        }
        container.setCacheLevel(configuration.getCacheLevel());
        container.setExceptionListener(configuration.getExceptionListener());
        container.setMaxMessagesToProcess(configuration.getMaxMessagesToProcess());
        container.setCacheLevel(configuration.getCacheLevel());
        container.setMaxReceivers(configuration.getMaxReceivers());
        container.setMinReceivers(configuration.getMinReceivers());
        container.setRecoveryInterval(configuration.getRecoveryInterval());
        container.setIdleLimit(configuration.getIdleLimit());
        container.setReceiveTimeout(configuration.getReceiveTimeout());
        container.setRecoveryInterval(configuration.getRecoveryInterval());
// TODO additional configuration        
//        container.setClientId();
//        container.setDurable();
//        container.setDurableSubscriptionName();
//        container.setLocalDelivery();
        if (started) {
            container.initialize();
            containers.put(serviceUri, container);
            monitor.registerListener(serviceUri);
        }
    }

    public void unregister(URI serviceUri) throws JMSException {
        AdaptiveMessageContainer container = containers.remove(serviceUri);
        if (container != null) {
            container.shutdown();
            monitor.unRegisterListener(serviceUri);
        }
    }

}
